@@ -67,35 +67,42 @@ class DAO(object):
         Session = scoped_session(session_factory)
 
         # todo add sources
+        params = {'doi': doi, }
 
         session = Session()
-        pub = session.query(Publication).filter_by(doi=doi).all()
-
+        # pub = session.query(Publication).filter_by(doi=doi).all()
+        p = text("""SELECT * FROM publication WHERE doi=:doi""")
+        p = p.bindparams(bindparam('doi'))
+        resultproxy = session.execute(p, params)
+        pub = [dict(row) for row in resultproxy]
         result = None
+
         if pub:
-            params = {'doi': doi, }
 
-            a = text("""SELECT name FROM "PublicationAuthor" as p
-                        JOIN "Author" as a on (a.id = p."authorId")
-                        WHERE p."publicationDoi"=:doi""")
+            a = text("""SELECT name FROM publication_author as p
+                        JOIN author as a on (a.id = p.author_id)
+                        WHERE p.publication_doi=:doi""")
             a = a.bindparams(bindparam('doi'))
-            authors = session.execute(a, params).fetchall()
+            resultproxy = session.execute(a, params)
+            authors = [dict(row) for row in resultproxy]
 
-            f = text("""SELECT name FROM "PublicationFieldOfStudy" as p
-                        JOIN "FieldOfStudy" as a on (a.id = p."fieldOfStudyId")
-                        WHERE p."publicationDoi"=:doi""")
+            f = text("""SELECT name FROM publication_field_of_study as p
+                        JOIN field_of_study as a on (a.id = p.field_of_study_id)
+                        WHERE p.publication_doi=:doi""")
             f = f.bindparams(bindparam('doi'))
-            fos = session.execute(f, params).fetchall()
+            resultproxy = session.execute(f, params)
+            fos = [dict(row) for row in resultproxy]
 
-            s = text("""SELECT title, url FROM "PublicationSource" as p
-                        JOIN "Source" as a on (a.id = p."sourceId")
-                        WHERE p."publicationDoi"=:doi""")
+            s = text("""SELECT title, url FROM publication_source as p
+                        JOIN source as a on (a.id = p.source_id)
+                        WHERE p.publication_doi=:doi""")
             s = s.bindparams(bindparam('doi'))
-            sources = session.execute(s, params).fetchall()
+            resultproxy = session.execute(s, params)
+            sources = [dict(row) for row in resultproxy]
 
-            result = DAO.object_as_dict(pub)
+            result = pub[0]
             result['authors']: authors
-            result['fieldsOfStudy']: fos
+            result['fields_of_study']: fos
             result['source_id']: sources
 
         session.close()
@@ -107,11 +114,11 @@ class DAO(object):
         session = Session()
 
         publication = Publication(doi=publication_data['doi'], type=publication_data['type'],
-                                  pubDate=publication_data['pubDate'], year=publication_data['year'],
+                                  pub_date=publication_data['pub_date'], year=publication_data['year'],
                                   publisher=publication_data['publisher'],
-                                  citationCount=publication_data['citationCount'],
+                                  citation_count=publication_data['citation_count'],
                                   title=publication_data['title'],
-                                  normalizedTitle=publication_data['normalizedTitle'],
+                                  normalized_title=publication_data['normalized_title'],
                                   abstract=publication_data['abstract'])
         publication = self.save_if_not_exist(session, publication, Publication, {'doi': publication.doi})
 
@@ -121,12 +128,12 @@ class DAO(object):
 
         authors = publication_data['authors']
         for author_data in authors:
-            author = Author(name=author_data['name'], normalizedName=author_data['normalizedName'])
+            author = Author(name=author_data['name'], normalized_name=author_data['normalized_name'])
 
-            author = self.save_if_not_exist(session, author, Author, {'normalizedName': author.normalizedName})
+            author = self.save_if_not_exist(session, author, Author, {'normalized_name': author.normalized_name})
             if author.id:
-                publication_authors = PublicationAuthor(**{'authorId': author.id, 'publicationDoi': publication.doi})
-                self.save_if_not_exist(session, publication_authors, PublicationAuthor, {'authorId': author.id, 'publicationDoi': publication.doi})
+                publication_authors = PublicationAuthor(**{'authorId': author.id, 'publication_doi': publication.doi})
+                self.save_if_not_exist(session, publication_authors, PublicationAuthor, {'authorId': author.id, 'publication_doi': publication.doi})
 
         if 'source_id' in publication_data:
             sources = publication_data['source_id']
@@ -134,18 +141,25 @@ class DAO(object):
                 source = Source(title=sources_data['title'], url=sources_data['url']) # todo no doi url ?
                 source = self.save_if_not_exist(session, source, Source, {'title': source.title})
                 if source.id:
-                    publication_sources = PublicationSource(**{'sourceId': source.id, 'publicationDoi': publication.doi})
-                    self.save_if_not_exist(session, publication_sources, PublicationSource, {'sourceId': source.id, 'publicationDoi': publication.doi})
+                    publication_sources = PublicationSource(**{'sourceId': source.id, 'publication_doi': publication.doi})
+                    self.save_if_not_exist(session, publication_sources, PublicationSource, {'sourceId': source.id, 'publication_doi': publication.doi})
 
-        if 'fieldsOfStudy' in publication_data:
-            fields_of_study = publication_data['fieldsOfStudy']
+        if 'fields_of_study' in publication_data:
+            fields_of_study = publication_data['fields_of_study']
             for fos_data in fields_of_study:
-                fos = FieldOfStudy(name=fos_data['name'], normalizedName=fos_data['normalizedName'])
-                fos.level = 2
-                fos = self.save_if_not_exist(session, fos, FieldOfStudy, {'normalizedName': fos.normalizedName})
+                if 'level' not in fos_data:
+                    fos_data['level'] = 2
+                fos = FieldOfStudy(name=fos_data['name'], normalized_name=fos_data['normalized_name'], level=fos_data['level'])
+                fos = self.save_if_not_exist(session, fos, FieldOfStudy, {'normalized_name': fos.normalized_name})
+
+                # check if we need an overwrite
+                if fos_data['level'] < 2 and fos.level == 2:
+                    fos.level = fos_data['level']
+                    DAO.save_object(session, fos)
+
                 if fos.id:
-                    publication_fos = PublicationFieldOfStudy(**{'fieldOfStudyId': fos.id, 'publicationDoi': publication.doi})
-                    self.save_if_not_exist(session, publication_fos, PublicationFieldOfStudy, {'fieldOfStudyId': fos.id, 'publicationDoi': publication.doi})
+                    publication_fos = PublicationFieldOfStudy(**{'fieldOfStudyId': fos.id, 'publication_doi': publication.doi})
+                    self.save_if_not_exist(session, publication_fos, PublicationFieldOfStudy, {'fieldOfStudyId': fos.id, 'publication_doi': publication.doi})
 
         session.close()
         return publication
@@ -163,38 +177,7 @@ class DAO(object):
         session_factory = sessionmaker(bind=self.engine)
         Session = scoped_session(session_factory)
         session = Session()
-        event_data['sourceId'] = 'twitter'  # todo
-        if 'location' not in event_data['subj']['processed']:
-            event_data['subj']['processed']['location'] = 'unknown'
-        if 'contains_abstract' not in event_data['subj']['processed']:
-            event_data['subj']['processed']['contains_abstract'] = 0
-        if 'sentiment' not in event_data['subj']['processed']:
-            event_data['subj']['processed']['sentiment'] = 0
-        if 'name' not in event_data['subj']['processed']:
-            event_data['subj']['processed']['name'] = 'unknown'
-
-        discussion_data = DiscussionData(
-            publicationDoi=event_data['obj']['data']['doi'],
-            createdAt=event_data['timestamp'],
-            score=event_data['subj']['processed']['score'],
-            timeScore=event_data['subj']['processed']['time_score'],
-            typeScore=event_data['subj']['processed']['type_score'],
-            userScore=event_data['subj']['processed']['user_score'],
-            language=event_data['subj']['data']['lang'],
-            source=event_data['subj']['data']['source'],
-            abstractDifference=event_data['subj']['processed']['contains_abstract'],
-            length=event_data['subj']['processed']['length'],
-            questions=event_data['subj']['processed']['question_mark_count'],
-            exclamations=event_data['subj']['processed']['exclamation_mark_count'],
-            type=event_data['subj']['processed']['tweet_type'],
-            sentiment=event_data['subj']['processed']['sentiment'],
-            subjId=event_data['subj']['alternative-id'],
-            followers=event_data['subj']['processed']['followers'],
-            botScore=event_data['subj']['processed']['bot_rating'],
-            authorName=event_data['subj']['processed']['name'],  # make this a table?
-            authorLocation=event_data['subj']['processed']['location'],
-            sourceId=event_data['sourceId'])
-        self.save_object(session, discussion_data)
+        publication_doi = event_data['obj']['data']['doi']
 
         if 'context_annotations' in event_data['subj']['data']:
             context_entity = event_data['subj']['data']['context_annotations']
@@ -203,9 +186,9 @@ class DAO(object):
                 entity = self.save_if_not_exist(session, entity, DiscussionEntity, {'entity': entity.entity})
 
                 publication_entity = DiscussionEntityData(
-                    **{'discussionDataId': discussion_data.id, 'discussionEntityId': entity.id})
+                    **{'publication_doi': publication_doi, 'discussion_entity_id': entity.id})
                 self.save_if_not_exist(session, publication_entity, DiscussionEntityData,
-                                       {'discussionDataId': discussion_data.id, 'discussionEntityId': entity.id})
+                                       {'publication_doi': publication_doi, 'discussion_entity_id': entity.id})
 
         if 'words' in event_data['subj']['processed']:
             words = event_data['subj']['processed']['words']
@@ -214,10 +197,10 @@ class DAO(object):
                 word = self.save_if_not_exist(session, word, DiscussionWord, {'word': word.word})
 
                 publication_words = DiscussionWordData(
-                    **{'discussionDataId': discussion_data.id, 'discussionWordId': word.id, 'count': words_data[1]})
+                    **{'publication_doi': publication_doi, 'discussion_word_id': word.id, 'count': words_data[1]})
                 self.save_object(session, publication_words)
                 self.save_if_not_exist(session, publication_words, DiscussionWordData,
-                                       {'discussionDataId': discussion_data.id, 'discussionWordId': word.id})
+                                       {'publication_doi': publication_doi, 'discussion_word_id': word.id})
 
         if 'entities' in event_data['subj']['data'] and 'hashtags' in event_data['subj']['data']['entities']:
             hashtags = event_data['subj']['data']['entities']['hashtags']
@@ -226,10 +209,10 @@ class DAO(object):
                 hashtag = self.save_if_not_exist(session, hashtag, DiscussionHashtag, {'hashtag': hashtag.hashtag})
 
                 publication_h = DiscussionHashtagData(
-                    **{'discussionDataId': discussion_data.id, 'discussionHashtagId': hashtag.id})
+                    **{'publication_doi': publication_doi, 'discussion_hashtag_id': hashtag.id})
                 self.save_object(session, publication_h)
                 self.save_if_not_exist(session, publication_h, DiscussionHashtagData,
-                                       {'discussionDataId': discussion_data.id, 'discussionHashtagId': hashtag.id})
+                                       {'publication_doi': publication_doi, 'discussion_hashtag_id': hashtag.id})
 
         session.close()
-        return discussion_data
+        return True
